from django.conf.urls import url

from bulbs.super_features.viewsets import RelationViewSet

urlpatterns = [
    url(
        r'content/(?P<pk>\d+)/relations/?$',
        RelationViewSet.as_view(),
        name='content-relations'
    ),
]



"""API filters for the Contribution App."""
from django.utils import dateparse, timezone

from rest_framework import filters

from bulbs.content.filters import Published


class ESPublishedFilterBackend(filters.BaseFilterBackend):
    """
    Used for elasticsearch publish queries in reporting.

    Rounds values to their given date & final second.
    """

    def filter_queryset(self, request, queryset, view):
        """Apply the relevant behaviors to the view queryset."""
        start_value = self.get_start(request)
        if start_value:
            queryset = self.apply_published_filter(queryset, "after", start_value)
        end_value = self.get_end(request)
        if end_value:
            # Forces the end_value to be the last second of the date provided in the query.
            # Necessary currently as our Published filter for es only applies to gte & lte.
            queryset = self.apply_published_filter(queryset, "before", end_value)
        return queryset

    def apply_published_filter(self, queryset, operation, value):
        """
        Add the appropriate Published filter to a given elasticsearch query.

        :param queryset: The DJES queryset object to be filtered.
        :param operation: The type of filter (before/after).
        :param value: The date or datetime value being applied to the filter.
        """
        if operation not in ["after", "before"]:
            raise ValueError("""Publish filters only use before or after for range filters.""")
        return queryset.filter(Published(**{operation: value}))

    def get_start(self, request):
        start_value = self.get_date_datetime_param(request, "start")
        if start_value:
            return timezone.make_aware(start_value).astimezone(timezone.pytz.utc)

    def get_end(self, request):
        end_value = self.get_date_datetime_param(request, "end")
        if end_value:
            end_value += timezone.timedelta(days=1)
            end_value -= timezone.timedelta(seconds=1)
            return timezone.make_aware(end_value).astimezone(timezone.pytz.utc)

    def get_date_datetime_param(self, request, param):
        """Check the request for the provided query parameter and returns a rounded value.

        :param request: WSGI request object to retrieve query parameter data.
        :param param: the name of the query parameter.
        """
        if param in request.GET:
            param_value = request.GET.get(param, None)
            # Match and interpret param if formatted as a date.
            date_match = dateparse.date_re.match(param_value)
            if date_match:
                return timezone.datetime.combine(
                    dateparse.parse_date(date_match.group(0)), timezone.datetime.min.time()
                )
            datetime_match = dateparse.datetime_re.match(param_value)
            if datetime_match:
                return timezone.datetime.combine(
                    dateparse.parse_datetime(datetime_match.group(0)).date(),
                    timezone.datetime.min.time()
                )
        return None


class StartEndFilterBackend(filters.BaseFilterBackend):

    def filter_queryset(self, request, queryset, view):
        start = request.QUERY_PARAMS.get("start", None)
        if start:
            start_date = dateparse.parse_date(start)
            queryset = self.filter_start(queryset, view, start_date)
        end = request.QUERY_PARAMS.get("end", None)
        if end:
            end_date = dateparse.parse_date(end)
            queryset = self.filter_end(queryset, view, end_date)
        return queryset

    def filter_start(self, queryset, view, start_date):
        start_fields = getattr(view, "start_fields", None)
        if start_fields:
            _filters = {}
            for field_name in start_fields:
                _filters = {"{}__gte".format(field_name): start_date}
        return queryset.filter(**_filters)

    def filter_end(self, queryset, view, end_date):
        end_fields = getattr(view, "end_fields", None)
        if end_fields:
            _filters = {}
            for field_name in end_fields:
                _filters = {"{}__lte".format(field_name): end_date}
        return queryset.filter(**_filters)



from django.conf import settings as user_settings
from django.utils.functional import LazyObject

from bulbs.conf import defaults


class Settings(object):
    pass


class LazySettings(LazyObject):
    def _setup(self):
        self._wrapped = Settings()
        for obj in (defaults, user_settings):
            for attr in dir(obj):
                if attr == attr.upper():
                    setattr(self, attr, getattr(obj, attr))


settings = LazySettings()



import time

from django.conf import settings
from django.template.loader import render_to_string
from django.template.base import TemplateDoesNotExist
from django.core.exceptions import ObjectDoesNotExist

from bulbs.utils import vault
from bulbs.instant_articles.renderer import InstantArticleRenderer
from bulbs.instant_articles.transform import transform

import requests
import logging
from celery import shared_task

logger = logging.getLogger(__name__)


@shared_task(default_retry_delay=5)
def index(content_type_id, pk, refresh=False):
    from django.contrib.contenttypes.models import ContentType
    content_type = ContentType.objects.get_for_id(content_type_id)
    obj = content_type.model_class().objects.get(id=pk)
    obj.index(refresh=refresh)


@shared_task(default_retry_delay=5)
def index_content_contributions(content_pk):
    from bulbs.contributions.models import Contribution
    for contribution in Contribution.objects.filter(content__pk=content_pk):
        contribution.save()


@shared_task(default_retry_delay=5)
def index_content_report_content_proxy(content_pk):
    from bulbs.contributions.models import ReportContent
    try:
        proxy = ReportContent.reference.get(id=content_pk)
        proxy.index()
    except ObjectDoesNotExist:
        pass


@shared_task(default_retry_delay=5)
def index_feature_type_content(featuretype_pk):
    from .models import FeatureType
    featuretype = FeatureType.objects.get(pk=featuretype_pk)
    for content in featuretype.content_set.all():
        content.index()


@shared_task(default_retry_delay=5)
def update_feature_type_rates(featuretype_pk):
    from bulbs.contributions.models import ContributorRole, FeatureTypeRate, FEATURETYPE

    roles = ContributorRole.objects.filter(payment_type=FEATURETYPE)

    for role in roles:
        existing_rates = FeatureTypeRate.objects.filter(
            feature_type_id=featuretype_pk,
            role_id=role.pk)

        if existing_rates.count() == 0:
            FeatureTypeRate.objects.create(
                rate=0,
                feature_type_id=featuretype_pk,
                role_id=role.pk)


def post_article(content, body, fb_page_id, fb_api_url, fb_token_path, fb_dev_mode, fb_publish):
    from .models import Content

    fb_access_token = vault.read(fb_token_path).get('authtoken')
    if fb_access_token is None:
        logger.error('Missing FB Auth Token in Vault.\n')
        return

    # Post article to instant article API
    post = requests.post(
        '{0}/{1}/instant_articles'.format(fb_api_url, fb_page_id),
        data={
            'access_token': fb_access_token,
            'html_source': body,
            'published': fb_publish,
            'development_mode': fb_dev_mode
        })

    if not post.ok:
        logger.error('''
                     Error in posting Instant Article.\n
                     Content ID: {0}\n
                     IA ID: {1}\n
                     Status Code: {2}
                     Request: {3}'''.format(content.id,
                                            content.instant_article_id,
                                            post.status_code,
                                            post.__dict__))
        return

    # Poll for status of article
    response = ""
    while response != "SUCCESS":
        time.sleep(1)

        status = requests.get('{0}/{1}?access_token={2}'.format(
            fb_api_url,
            post.json().get('id'),
            fb_access_token
        ))

        # log errors
        if not status.ok or status.json().get('status') == "ERROR":
            logger.error('''
                Error in getting status of Instant Article.\n
                Content ID: {0}\n
                IA ID: {1}\n
                Status Code: {2}
                Request: {3}'''.format(content.id,
                                       content.instant_article_id,
                                       status.status_code,
                                       status.__dict__))
            return

        response = status.json().get('status')

    # build URL
    base = getattr(settings, 'WWW_URL')
    if not base.startswith("http"):
        base = "http://" + base

    canonical_url = "{0}{1}".format(base, content.get_absolute_url())
    canon = requests.get('{0}?id={1}&fields=instant_article&access_token={2}'.format(
        fb_api_url,
        canonical_url,
        fb_access_token))

    if not canon.ok:
        logger.error('''
            Error in getting article ID of Instant Article.\n
            Content ID: {0}\n
            Status Code: {1}
            Request: {2}'''.format(content.id,
                                   canon.status_code,
                                   canon.__dict__))
        return

    # set instant_article_id to response id
    Content.objects.filter(pk=content.id).update(
        instant_article_id=canon.json().get('instant_article').get('id'))


def delete_article(content, fb_api_url, fb_token_path):
    fb_access_token = vault.read(fb_token_path).get('authtoken')
    if fb_access_token is None:
        logger.error('Missing FB Auth Token in Vault.\n')
        return

    delete = requests.delete('{0}/{1}?access_token={2}'.format(
        fb_api_url,
        content.instant_article_id,
        fb_access_token
    ))

    if not delete.ok:
        logger.error('''
                     Error in deleting Instant Article.\n
                     Content ID: {0}\n
                     IA ID: {1}\n
                     Status Code: {2}
                     Request: {3}'''.format(content.id,
                                            content.instant_article_id,
                                            delete.status_code,
                                            delete.__dict__))
    else:
        status = delete.json().get('success')
        if bool(status) is not True:
            logger.error('''
                Error in deleting Instant Article.\n
                Content ID: {0}\n
                IA ID: {1}\n
                Error: {2}'''.format(content.id,
                                     content.instant_article_id,
                                     delete.json()))


@shared_task(default_retry_delay=5, time_limit=300)
def post_to_instant_articles_api(content_pk):
    from .models import Content
    content = Content.objects.get(pk=content_pk)

    fb_page_id = getattr(settings, 'FACEBOOK_PAGE_ID', None)
    fb_api_url = getattr(settings, 'FACEBOOK_API_BASE_URL', None)
    fb_token_path = getattr(settings, 'FACEBOOK_TOKEN_VAULT_PATH', None)
    fb_dev_mode = 'true' if getattr(settings, 'FACEBOOK_API_DEVELOPMENT_MODE', False) else 'false'
    fb_publish = 'true' if getattr(settings, 'FACEBOOK_API_PUBLISH_ARTICLE', False) else 'false'
    should_post = getattr(settings, 'FACEBOOK_POST_TO_IA', False)

    if not fb_page_id or not fb_api_url or not fb_token_path:
        logger.error('''
                     Error in Django Settings.\n
                     FACEBOOK_PAGE_ID: {0}\n
                     FACEBOOK_API_BASE_URL: {1}\n
                     FACEBOOK_TOKEN_VAULT_PATH: {2}'''.format(fb_page_id,
                                                              fb_api_url,
                                                              fb_token_path))
        return

    # if feature type is IA approved & content is published
    feature_type = getattr(content, 'feature_type', None)
    if feature_type and feature_type.instant_article and content.is_published:
        # render page source
        context = {
            'content': content,
            'absolute_uri': getattr(settings, 'WWW_URL'),
            'transformed_body': transform(
                getattr(content, 'body', ''),
                InstantArticleRenderer())
        }
        try:
            source = render_to_string(
                'instant_article/_instant_article.html', context
            )
        except TemplateDoesNotExist:
            source = render_to_string(
                'instant_article/base_instant_article.html', context
            )

        if should_post:
            post_article(
                content,
                source,
                fb_page_id,
                fb_api_url,
                fb_token_path,
                fb_dev_mode,
                fb_publish)

    # if article is being unpublished, delete it from IA API
    elif not content.is_published and content.instant_article_id:
        if should_post:
            delete_article(
                content,
                fb_api_url,
                fb_token_path)



from rest_framework import serializers

from bulbs.utils.fields import RichTextField
from bulbs.utils.data_serializers import CopySerializer, EntrySerializer, BaseEntrySerializer
from .fields import ColorField


class XYEntrySerializer(BaseEntrySerializer):
    title = RichTextField(required=False, field_size="short")
    copy_x = RichTextField(field_size="long")
    copy_y = RichTextField(field_size="long")


class ComparisonKeySerializer(serializers.Serializer):
    title = RichTextField(required=False, field_size="short")
    color = ColorField(required=False)
    initial = serializers.CharField()


class ComparisonSerializer(serializers.Serializer):
    key_x = ComparisonKeySerializer(required=False)
    key_y = ComparisonKeySerializer(required=False)
    entries = XYEntrySerializer(required=False, many=True, child_label="entry")


class ListInfographicDataSerializer(serializers.Serializer):
    is_numbered = serializers.BooleanField(default=False)
    entries = EntrySerializer(many=True, required=False, child_label="entry")


class ProConSerializer(serializers.Serializer):
    body = RichTextField(required=False, field_size="long")
    pro = CopySerializer(required=False, many=True)
    con = CopySerializer(required=False, many=True)


class StrongSideWeakSideSerializer(serializers.Serializer):
    body = RichTextField(required=False, field_size="long")
    strong = CopySerializer(required=False, many=True)
    weak = CopySerializer(required=False, many=True)


class TimelineSerializer(serializers.Serializer):
    entries = EntrySerializer(many=True, required=False, child_label="entry")



from django.conf.urls import patterns, url

from .views import (
    instant_article_rss, instant_article, instant_article_analytics, instant_article_ad
)

urlpatterns = patterns(
    "",
    url(r"^instant_articles/feed", instant_article_rss, name="instant_articles"),
    url(r"^instant_article/(?P<pk>\d+)/?$", instant_article, name="instant_article"),
    url(
        r"^instant_article/(?P<pk>\d+)/analytics$",
        instant_article_analytics,
        name="instant_article_analytics"
    ),
    url(r"^instant_article/(?P<pk>\d+)/ad$", instant_article_ad, name="instant_article_ad"),
)



import vcr
import os

from django.conf import settings


def make_vcr(test_path, record_mode='once'):
    base_dir = os.path.dirname(os.path.realpath(test_path))
    cassette_dir = os.path.join(base_dir, 'test_data', 'cassettes')

    return vcr.VCR(
        cassette_library_dir=cassette_dir,
        ignore_hosts=settings.ES_CONNECTIONS['default']['hosts'],
        record_mode=record_mode,
        filter_post_data_parameters=['access_token'],
        filter_query_parameters=['access_token']
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contributions', '0002_auto_20151001_1201'),
    ]

    operations = [
        migrations.AlterField(
            model_name='freelanceprofile',
            name='is_manager',
            field=models.BooleanField(default=False),
        ),
    ]



from djbetty.serializers import ImageFieldSerializer
from rest_framework import serializers

from .models import VideohubVideo


class VideohubVideoSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField()
    image = ImageFieldSerializer(allow_null=True, default={})
    hub_url = serializers.CharField(source="get_hub_url", read_only=True)
    embed_url = serializers.CharField(source="get_embed_url", read_only=True)
    api_url = serializers.CharField(source="get_api_url", read_only=True)
    channel_id = serializers.IntegerField()

    class Meta:
        model = VideohubVideo

    def save(self, **kwargs):
        """
        Save and return a list of object instances.
        """
        validated_data = [
            dict(list(attrs.items()) + list(kwargs.items()))
            for attrs in self.validated_data
        ]

        if "id" in validated_data:
            ModelClass = self.Meta.model

            try:
                self.instance = ModelClass.objects.get(id=validated_data["id"])
            except ModelClass.DoesNotExist:
                pass

        return super(VideohubVideoSerializer, self).save(**kwargs)

    def to_internal_value(self, data):
        # Channel info passed as nested object, but we just store integer ID
        channel = data.get('channel')
        if channel and 'id' in channel:
            data['channel_id'] = channel['id']
        return super(VideohubVideoSerializer, self).to_internal_value(data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import json_field.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TargetingOverride',
            fields=[
                ('url', models.URLField(primary_key=True, serialize=False)),
                ('targeting', json_field.fields.JSONField(default='null', help_text='Enter a valid JSON object')),
            ],
        ),
    ]



from django import template
from bulbs.utils.methods import get_overridable_template_name

register = template.Library()

template_name = get_overridable_template_name(
    'homepage_player/bulbs_homepage_player.html',
    'homepage_player/homepage_player_override.html')


@register.inclusion_tag(template_name, takes_context=True)
def homepage_player_partial(context, twitter_handle, share_message, share_style, videos=[]):
    context['videos'] = videos
    context['twitter_handle'] = twitter_handle
    context['share_message'] = share_message
    context['share_style'] = share_style
    return context



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sections', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='section',
            name='description',
            field=models.TextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='section',
            name='embed_code',
            field=models.TextField(blank=True, default=''),
        ),
    ]



from rest_framework import serializers

from djbetty.serializers import ImageFieldSerializer

from bulbs.utils.data_serializers import EntrySerializer


class GuideToChildSerializer(serializers.Serializer):
    entries = EntrySerializer(many=True, required=False)


class GuideToParentSerializer(serializers.Serializer):
    sponsor_text = serializers.CharField(required=False)
    sponsor_image = ImageFieldSerializer(required=False)



class SuperFeatureMixin(object):
    def get_absolute_url(self):
        if self.parent:
            return self.parent.get_absolute_url() + "/{}".format(self.slug)

        return "/{}".format(self.slug)

    @property
    def is_indexed(self):
        if self.parent:
            return False

        return self.indexed

    @property
    def is_parent(self):
        return self.parent is None

    @property
    def is_child(self):
        return self.parent is not None



import argparse

from django.core.management.base import BaseCommand
from django.utils import timezone

from bulbs.content.models import Content
from bulbs.contributions.models import ContributorRole


def valid_date(s):
    try:
        return timezone.datetime.strptime(s, "%Y-%m-%d")
    except ValueError:
        msg = "Not a valid date: '{0}'.".format(s)
        raise argparse.ArgumentTypeError(msg)


class Command(BaseCommand):

    help = "add missing contributions from authors of the previous month."
    default_role = ContributorRole.objects.first()
    count = 0

    def add_arguments(self, parser):
        parser.add_argument(
            '-start',
            '--start',
            help='The Start Date - format YYYY-MM-DD',
            required=False,
            type=valid_date
        )
        parser.add_argument(
            '-end',
            '--end',
            help='The End Date - format YYYY-MM-DD',
            required=False,
            type=valid_date
        )

    def get_date_range(self):
        now = timezone.now()
        month_start = timezone.datetime(
            year=now.year,
            month=now.month,
            day=1
        )
        return month_start, now

    def get_content_list(self, start, end):
        return Content.objects.filter(published__range=(start, end))

    def process_content_contributions(self, content):
        for author in content.authors.all():
            contribution_qs = content.contributions.filter(contributor=author)
            if not contribution_qs.exists():
                content.contributions.create(
                    contributor=author,
                    role=self.default_role
                )
                self.count += 1

    def handle(self, *args, **kwargs):
        if not self.default_role:
            self.stdout.write('No role available for backfill')
            return

        default_start, default_end = self.get_date_range()
        start = kwargs['start'] if kwargs['start'] else default_start
        end = kwargs['end'] if kwargs['end'] else default_end
        content_qs = self.get_content_list(start, end)
        for content in content_qs.all():
            self.process_content_contributions(content)
        self.stdout.write(
            '{0} contributions created between {1} - {2}'.format(self.count, start, end)
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('content', '0007_content_evergreen'),
    ]

    operations = [
        migrations.AddField(
            model_name='content',
            name='template_choice',
            field=models.IntegerField(choices=[(0, 'Default')], default=0),
        ),
    ]



from importlib import import_module

from django.conf import settings

from .enum import InfographicType
from .data_serializers import (
    ComparisonSerializer, ListInfographicDataSerializer, ProConSerializer,
    StrongSideWeakSideSerializer, TimelineSerializer
)


def get_infographics_serializer():
    module = getattr(settings, "INFOGRAPHICS_MODULE", None)
    serializer = getattr(settings, "INFOGRAPHICS_SERIALIZER", None)
    if serializer is None or module is None:
        from .serializers import InfographicSerializer
        return InfographicSerializer
    else:
        mod = import_module(module)
        return getattr(mod, serializer)


def get_data_serializer(infographic_type):
    serializer = {
        InfographicType.LIST: ListInfographicDataSerializer,
        InfographicType.TIMELINE: TimelineSerializer,
        InfographicType.STRONGSIDE_WEAKSIDE: StrongSideWeakSideSerializer,
        InfographicType.PRO_CON: ProConSerializer,
        InfographicType.COMPARISON: ComparisonSerializer
    }.get(infographic_type, None)
    if serializer is None:
        raise KeyError(
            """The requested Infographic does not have a configured serializer."""
        )
    return serializer



from rest_framework import serializers

from .validators import ColorValidator


class ColorField(serializers.CharField):

    default_error_messages = {
        'invalid': ('Enter a valid hex value. e.g., #F0F8FF ')
    }

    def __init__(self, **kwargs):
        super(ColorField, self).__init__(**kwargs)
        self.validators.append(
            ColorValidator(message=self.error_messages['invalid'])
        )
